PROGRAM CPD9;

DEVICE = mega2560;

{$I mega2560_Register.INC}


CONST

  // TaktFrequenz in KHz
  TaktFrequenz : WORD = 16000;

  // bei 16,000 MHz ->
  UBRR_VAL : Byte = 5;


VAR

  // Steuerleitungen Ausgang (ROT)
  Z80_M1_A[PortC,0] : bit;
  Z80_M1_E[PinC,0] : bit;
  Z80_M1_D[DDRC,0] : bit;

  Z80_MREQ_A[PortC,1] : bit;
  Z80_MREQ_E[PinC,1] : bit;
  Z80_MREQ_D[DDRC,1] : bit;

  Z80_IORQ_A[PortC,2] : bit;
  Z80_IORQ_E[PinC,2] : bit;
  Z80_IORQ_D[DDRC,2] : bit;

  Z80_RD_A[PortC,3] : bit;
  Z80_RD_E[PinC,3] : bit;
  Z80_RD_D[DDRC,3] : bit;

  Z80_WR_A[PortC,4] : bit;
  Z80_WR_E[PinC,4] : bit;
  Z80_WR_D[DDRC,4] : bit;

  Z80_RFSH_A[PortC,5] : bit;
  Z80_RFSH_E[PinC,5] : bit;
  Z80_RFSH_D[DDRC,5] : bit;

  Z80_HALT_A[PortC,6] : bit;
  Z80_HALT_E[PinC,6] : bit;
  Z80_HALT_D[DDRC,6] : bit;

  Z80_BUSACK_A[PortC,7] : bit;
  Z80_BUSACK_E[PinC,7] : bit;
  Z80_BUSACK_D[DDRC,7] : bit;


  // Steuerleitungen Eingang (GELB)
  Z80_CLK_A[PortL,0] : bit;
  Z80_CLK_E[PinL,0] : bit;
  Z80_CLK_D[DDRL,0] : bit;

  Z80_RESET_A[PortL,2] : bit;
  Z80_RESET_E[PinL,2] : bit;
  Z80_RESET_D[DDRL,2] : bit;

  Z80_WAIT_A[PortL,4] : bit;
  Z80_WAIT_E[PinL,4] : bit;
  Z80_WAIT_D[DDRL,4] : bit;

  Z80_NMI_A[PortL,6] : bit;
  Z80_NMI_E[PinL,6] : bit;
  Z80_NMI_D[DDRL,6] : bit;

  Z80_INT_A[PortG,0] : bit;
  Z80_INT_E[PinG,0] : bit;
  Z80_INT_D[DDRG,0] : bit;

  Z80_BUSRQ_A[PortG,2] : bit;
  Z80_BUSRQ_E[PinG,2] : bit;
  Z80_BUSRQ_D[DDRG,2] : bit;


  // Steuerleitungen RAM (BLAU)
  RAM_WE_A[PortD,7] : bit;
  RAM_WE_E[PinD,7] : bit;
  RAM_WE_D[DDRD,7] : bit;

  RAM_RD_A[PortG,1] : bit;
  RAM_RD_E[PinG,1] : bit;
  RAM_RD_D[DDRG,1] : bit;

  RAM_CE_A[PortL,7] : bit;
  RAM_CE_E[PinL,7] : bit;
  RAM_CE_D[DDRL,7] : bit;


  // zusaetzliche LEDs
  LED_BL3_A[PortH,6] : bit;
  LED_BL3_D[DDRH,6] : bit;

  LED_BL4_A[PortB,4] : bit;
  LED_BL4_D[DDRB,4] : bit;

  LED_BL5_A[PortB,5] : bit;
  LED_BL5_D[DDRB,5] : bit;

  LED_BL6_A[PortB,6] : bit;
  LED_BL6_D[DDRB,6] : bit;

  LED_BL7_D[DDRB,7] : bit;
  LED_BL7_A[PortB,7] : bit;

  // die drei Taster
  Taster_01_D[DDRH,3] : bit;
  Taster_01_E[PinH,3] : bit;

  Taster_02_D[DDRH,4] : bit;
  Taster_02_E[PinH,4] : bit;

  Taster_03_D[DDRH,5] : bit;
  Taster_03_E[PinH,5] : bit;


  CLK_Counter : Byte;

  RAM_Read_aktiv : Boolean;




{$I SYSTEM.INC}

{$I RAM.INC}



procedure InitPorts;
begin

  // PORT A -> D0..D7
  DDRA:=  %00000000;

  // PORT F -> A0..A7
  DDRF:=  %00000000;

  // PORT K -> A8..A15
  DDRK:=  %00000000;

  // PORT C -> Steuerleitungen Eingang (Rot)
  DDRC:=  %00000000;

  // Steuerleitungen Eingang (GELB)
  // Takt auf 0
  SetBit(Z80_CLK_D);
  ClearBit(Z80_CLK_A);

  // /RESET auf 1 (inaktiv)
  SetBit(Z80_RESET_D);
  SetBit(Z80_RESET_A);

  // /WAIT auf 1 (inaktiv)
  SetBit(Z80_WAIT_D);
  SetBit(Z80_WAIT_A);

  // /NMI auf 1 (inaktiv)
  SetBit(Z80_NMI_D);
  SetBit(Z80_NMI_A);

  // /INT auf 1 (inaktiv)
  SetBit(Z80_INT_D);
  SetBit(Z80_INT_A);

  // /BUSRQ auf 1 (inaktiv)
  SetBit(Z80_BUSRQ_D);
  SetBit(Z80_BUSRQ_A);

  // Steuerleitungen RAM (BLAU)
  // /RAM_WE auf 1 (inaktiv)
  SetBit(RAM_WE_D);
  SetBit(RAM_WE_A);

  // /RAM_RD auf 1 (inaktiv)
  SetBit(RAM_RD_D);
  SetBit(RAM_RD_A);

  // /RAM_CE auf 1 (inaktiv)
  SetBit(RAM_CE_D);
  SetBit(RAM_CE_A);

  // zusaetzliche LEDs
  // LED_BL3 aus
  SetBit(LED_BL3_D);
  ClearBit(LED_BL3_A);

  // LED_BL4 aus
  SetBit(LED_BL4_D);
  ClearBit(LED_BL4_A);

  // LED_BL5 aus
  SetBit(LED_BL5_D);
  ClearBit(LED_BL5_A);

  // LED_BL6 aus
  SetBit(LED_BL6_D);
  ClearBit(LED_BL6_A);

  // LED_BL7 aus
  SetBit(LED_BL7_D);
  ClearBit(LED_BL7_A);

  // Taster_01 auf Eingang
  ClearBit(Taster_01_D);

  // Taster_02 auf Eingang
  ClearBit(Taster_02_D);

  // Taster_03 auf Eingang
  ClearBit(Taster_03_D);

  RAM_Array[0]:=$C3;
  RAM_Array[1]:=$00;
  RAM_Array[2]:=$00;

end InitPorts;




procedure RAM_Read_einschalten;
var
  Data_Byte : Byte;
begin

  RAM_Read_aktiv:=True;

  // Daten auf Ausgang
  DDRA:=$FF;

  Data_Byte:=Lese_RAM;

  PortA:=Data_Byte;

end RAM_Read_einschalten;




procedure RAM_Read_ausschalten;
begin

  RAM_Read_aktiv:=False;

  // Daten auf Eingang
  DDRA:=$00;

  PortA:=$00;

end RAM_Read_ausschalten;



procedure Abfrage_MREQ_RD;
begin

  if BitLow(Z80_MREQ_E) then

    if BitLow(Z80_RD_E) then

      if RAM_Read_aktiv=False then

        RAM_Read_einschalten;

        SetBit(LED_BL5_A);
        wait_ms(100);
        ClearBit(LED_BL5_A);
        wait_ms(100);
        SetBit(LED_BL5_A);
        wait_ms(100);
        ClearBit(LED_BL5_A);
        wait_ms(100);
        SetBit(LED_BL5_A);
        wait_ms(100);
        ClearBit(LED_BL5_A);
        wait_ms(100);
        SetBit(LED_BL5_A);
        wait_ms(100);
        ClearBit(LED_BL5_A);

      endif; // if RAM_Read_aktiv=False then

    else

      if RAM_Read_aktiv=True then

        RAM_Read_ausschalten;

      endif;

    endif; // if BitLow(Z80_RD_E) then

  else

    if RAM_Read_aktiv=True then

      RAM_Read_ausschalten;

    endif;

  endif; // if BitLow(Z80_MREQ_E) then


end Abfrage_MREQ_RD;




BEGIN


  InitPorts;

  Init_RAM;



  wait_ms(100);
  ClearBit(Z80_RESET_A);
  SetBit(LED_BL6_A);


  for CLK_Counter:=1 to 100 do

    ClearBit(LED_BL6_A);
    ClearBit(Z80_CLK_A);
    wait_ms(1);

    SetBit(LED_BL6_A);
    SetBit(Z80_CLK_A);
    wait_ms(1);

  endfor;

  wait_ms(100);
  SetBit(Z80_RESET_A);
  ClearBit(LED_BL6_A);


  loop

    while BitLow(Taster_01_E) do
      SetBit(LED_BL7_A);
      SetBit(Z80_CLK_A);
      wait_ms(100);
    endwhile;

    Abfrage_MREQ_RD;

    while BitHigh(Taster_01_E) do
      ClearBit(LED_BL7_A);
      ClearBit(Z80_CLK_A);
      wait_ms(100);
    endwhile;

    Abfrage_MREQ_RD;

  endloop;



END CPD9.    
